#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "avinfo.hpp"
#include <iostream>

extern bool verbose;
extern std::string sep;

std::string find_codec_name( CodecID id )
{

	std::string codec_name;

	if(id)
	{
		AVCodec *Codec = avcodec_find_decoder( id );
		codec_name = Codec->name;
	}
	else
		codec_name = "unknown";

	return codec_name;

}

int get_avinfo( AVInfo *avinfo, std::string path )
{

	AVFormatContext	*FormatCtx;
	AVCodecContext	*VCodecCtx;
	AVCodecContext	*ACodecCtx;
	unsigned int i;
	int VideoStream, AudioStream;

	// Register all formats and codecs
	av_register_all();

	// Open video file
	if(av_open_input_file(&FormatCtx, path.c_str(), NULL, 0, NULL)!=0)
		return -1;//Couldn't open the file

	// Retrieve stream information
	if(av_find_stream_info(FormatCtx)<0)
		return -1;//Couldn't find stream info

	// Dump information about file
	if( verbose )
	{
		std::cout << sep;
		dump_format(FormatCtx, 0, path.c_str(), false);
	}

	// Find the first video stream
	VideoStream=-1;
	for(i=0; i<FormatCtx->nb_streams; i++)
		if(FormatCtx->streams[i]->codec->codec_type==CODEC_TYPE_VIDEO)
		{
			VideoStream=i;
			break;
		}
	if(VideoStream==-1)
		return -1;// Couldn't find any video stream

	// Find the first audio stream
	AudioStream=-1;
	for(i=0; i<FormatCtx->nb_streams; i++)
		if(FormatCtx->streams[i]->codec->codec_type==CODEC_TYPE_AUDIO)
		{
			AudioStream=i;
			break;
		}
	if(AudioStream==-1)
		return -1;// Couldn't find any audio stream

	// Get a pointer to the codec context for the video & audio streams
	VCodecCtx=FormatCtx->streams[VideoStream]->codec;
	ACodecCtx=FormatCtx->streams[AudioStream]->codec;

	// Start filing the class with information
	avinfo->duration	= FormatCtx->duration / 1000000LL;

	avinfo->width		= VCodecCtx->width;
	avinfo->height		= VCodecCtx->height;
	avinfo->aspect		= av_q2d(VCodecCtx->sample_aspect_ratio) * VCodecCtx->width / VCodecCtx->height;
	avinfo->vcodec		= find_codec_name( VCodecCtx->codec_id );
	avinfo->vbitrate	= VCodecCtx->bit_rate;
	avinfo->framerate	= 1 / av_q2d(VCodecCtx->time_base);

	avinfo->acodec		= find_codec_name( ACodecCtx->codec_id );
	avinfo->abitrate	= ACodecCtx->bit_rate;
	avinfo->samplerate	= ACodecCtx->sample_rate;

	// Close the video file
	av_close_input_file(FormatCtx);

	return 0;
}

unsigned int AVInfo::get_duration()
{
	return duration;
}

unsigned int AVInfo::get_width()
{
	return width;
}

unsigned int AVInfo::get_height()
{
	return height;
}

double AVInfo::get_aspect()
{
	return aspect;
}

std::string AVInfo::get_vcodec()
{
	return vcodec;
}

unsigned int AVInfo::get_vbitrate()
{
	return vbitrate;
}

float AVInfo::get_framerate()
{
	return framerate;
}

std::string AVInfo::get_acodec()
{
	return acodec;
}

unsigned int AVInfo::get_abitrate()
{
	return abitrate;
}

unsigned int AVInfo::get_samplerate()
{
	return samplerate;
}
