// Some pats of code come from ffmpeg.c witch is Copyrighted (c) 2000-2007 Fabrice Bellard, et al.
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "check.hpp"

#include <gtkmm/messagedialog.h>
extern "C" {
#include <avformat.h>
}
#include <list>
#include <iostream>

std::list<formats_t>	formats;
std::list<codecs_t>	codecs;

bool check_ffmpeg()
{

	if(system("ffmpeg &>/dev/null") == 127)
	{
		Glib::ustring message("<b>Can't find ffmpeg!</b>");
		Gtk::MessageDialog dialog( message, true, Gtk::MESSAGE_ERROR , Gtk::BUTTONS_CLOSE );
		dialog.set_secondary_text ("If ffmpeg is already installed, check that it's in your path.");
		dialog.run();
		return false;
	}
	else
		return true;

}

void get_formats()
{

	std::list<formats_t> list;

	av_register_all();

	AVInputFormat *ifmt;
	AVOutputFormat *ofmt;
	const char *last_name;

	last_name= "000";
	for(;;)
	{
		bool decode;
		bool encode;
		const char *name=NULL;
		const char *long_name=NULL;


		for(ofmt = first_oformat; ofmt != NULL; ofmt = ofmt->next)
		{
			if((name == NULL || strcmp(ofmt->name, name)<0) && strcmp(ofmt->name, last_name)>0)
			{
				name= ofmt->name;
				long_name= ofmt->long_name;
				encode=true;
			}
		}

		for(ifmt = first_iformat; ifmt != NULL; ifmt = ifmt->next)
		{
			if((name == NULL || strcmp(ifmt->name, name)<0) && strcmp(ifmt->name, last_name)>0)
			{
				name= ifmt->name;
				long_name= ifmt->long_name;
				encode=false;
			}
			if(name && strcmp(ifmt->name, name)==0)
				decode=true;
		}

		if(name==NULL)
			break;
		last_name= name;

		formats_t tmp;
		if(decode)
			tmp.decode = true;
		else
			tmp.decode = false;
		if(encode)
			tmp.encode = true;
		else
			tmp.encode = false;

		tmp.name = name;
		tmp.description = long_name;

		list.push_back (tmp);
	}

	formats = list;

}

void get_codecs()
{

	std::list<codecs_t> list;

	av_register_all();

	AVCodec *p, *p2;
	const char *last_name;

	last_name= "000";
	for(;;)
	{
		bool decode;
		bool encode;
		int cap=0;

		p2=NULL;
		for(p = first_avcodec; p != NULL; p = p->next)
		{
			if((p2==NULL || strcmp(p->name, p2->name)<0) && strcmp(p->name, last_name)>0)
			{
				p2= p;
				decode = false;
				encode = false;
				cap=0;
			}
			if(p2 && strcmp(p->name, p2->name)==0)
			{
				if(p->decode)
					decode=true;
				if(p->encode)
					encode=true;
				cap |= p->capabilities;
			}
		}
		if(p2==NULL)
			break;
		last_name= p2->name;

		codecs_t tmp;
		if(decode)
			tmp.decode = true;
		else
			tmp.decode = false;
		if(encode)
			tmp.encode = true;
		else
			tmp.encode = false;

		switch(p2->type)
		{
			case CODEC_TYPE_VIDEO:
				tmp.type = "Video";
				break;
			case CODEC_TYPE_AUDIO:
				tmp.type = "Audio";
				break;
			case CODEC_TYPE_SUBTITLE:
				tmp.type = "Subtitle";
				break;
			default:
				tmp.type = "Unknown";
				break;
		}

		tmp.name = p2->name;

		list.push_back (tmp);
	}

	codecs = list;

}

bool check_encodec (std::string codec_name)
{

	for(std::list<codecs_t>::iterator iter = codecs.begin(); iter != codecs.end(); iter++)
	{
		if(iter->name == codec_name && iter->encode)
			return true;
	}

	return false;

}

bool check_decodec (std::string codec_name)
{

	for(std::list<codecs_t>::iterator iter = codecs.begin(); iter != codecs.end(); iter++)
	{
		if(iter->name == codec_name && iter->decode)
			return true;
	}

	return false;

}

