#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "formatsdialog.hpp"
#include "check.hpp"
#include "ucompose.hpp"

#include <gtkmm/stock.h>

extern std::list<formats_t>	formats;
extern std::list<codecs_t>	codecs;

Formats::Formats()
{

	set_title ("Supported formats and codecs");

	set_default_size (300,400);
	set_has_separator (false);

	// Formats Tree
	m_ScrolledWindow_Formats.add (m_TreeView_Formats);
	m_ScrolledWindow_Formats.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC);

	m_refTreeModel_Formats = Gtk::ListStore::create (m_ModelColumns_Formats);
	m_TreeView_Formats.set_model (m_refTreeModel_Formats);
	m_TreeView_Formats.set_grid_lines (Gtk::TREE_VIEW_GRID_LINES_VERTICAL);

	m_TreeView_Formats.append_column ("Decode", m_ModelColumns_Formats.m_col_decode);
	m_TreeView_Formats.append_column ("Encode", m_ModelColumns_Formats.m_col_encode);
	m_TreeView_Formats.append_column ("Name", m_ModelColumns_Formats.m_col_name);
	m_TreeView_Formats.append_column ("Description", m_ModelColumns_Formats.m_col_description);

	m_TreeView_Formats.set_search_column(3);

	// Codecs Tree
	m_ScrolledWindow_Codecs.add (m_TreeView_Codecs);
	m_ScrolledWindow_Codecs.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC);

	m_refTreeModel_Codecs = Gtk::ListStore::create (m_ModelColumns_Codecs);
	m_TreeView_Codecs.set_model (m_refTreeModel_Codecs);
	m_TreeView_Codecs.set_grid_lines (Gtk::TREE_VIEW_GRID_LINES_VERTICAL);

	m_TreeView_Codecs.append_column ("Decode", m_ModelColumns_Codecs.m_col_decode);
	m_TreeView_Codecs.append_column ("Encode", m_ModelColumns_Codecs.m_col_encode);
	m_TreeView_Codecs.append_column ("Name", m_ModelColumns_Codecs.m_col_name);
	m_TreeView_Codecs.append_column ("Type", m_ModelColumns_Codecs.m_col_type);

	m_TreeView_Codecs.set_search_column(3);


	for(std::list<formats_t>::iterator iter = formats.begin(); iter != formats.end(); iter++)
	{
		Gtk::TreeModel::Row row = *(m_refTreeModel_Formats->append());

		row[m_ModelColumns_Formats.m_col_decode]	= iter->decode;
		row[m_ModelColumns_Formats.m_col_encode]	= iter->encode;
		row[m_ModelColumns_Formats.m_col_name]		= iter->name;
		row[m_ModelColumns_Formats.m_col_description]	= iter->description;
	}


	for(std::list<codecs_t>::iterator iter = codecs.begin(); iter != codecs.end(); iter++)
	{
		Gtk::TreeModel::Row row = *(m_refTreeModel_Codecs->append());

		row[m_ModelColumns_Codecs.m_col_decode]	= iter->decode;
		row[m_ModelColumns_Codecs.m_col_encode]	= iter->encode;
		row[m_ModelColumns_Codecs.m_col_type]	= iter->type;
		row[m_ModelColumns_Codecs.m_col_name]	= iter->name;
	}

	m_Notebook.append_page (m_ScrolledWindow_Formats, "Formats");
	m_Notebook.append_page (m_ScrolledWindow_Codecs, "Codecs");

	Gtk::VBox *pVBox = this->get_vbox();
	pVBox->pack_start (m_Notebook, Gtk::PACK_EXPAND_WIDGET);

	tree_children_t children_formats	= m_refTreeModel_Formats->children();
	tree_children_t children_codecs		= m_refTreeModel_Codecs->children();

	m_Label_Info.set_markup (String::ucompose ("<b>Supported formats:</b>\t%1\n<b>Supported codecs:</b>\t%2",
							children_formats.size(),
							children_codecs.size()));

	m_Label_Note.set_markup ("<span size='small'>" \
		"*Note: The names of encoders and decoders don't always match," \
		" so there are several cases where the above table shows encoder" \
		" only or decoder only entries even though both encoding and" \
		" decoding are supported for example, the h263 decoder corresponds" \
		" to the h263 and h263p encoders, for file formats it's even worse." \
		"</span>");
	m_Label_Note.set_line_wrap();
	m_Label_Note.set_sensitive (false);

	m_HBox1.pack_start (m_Label_Info, Gtk::PACK_SHRINK);
	m_HBox2.pack_start (m_Label_Note, Gtk::PACK_SHRINK);

	pVBox->pack_start (m_HBox1, Gtk::PACK_SHRINK);
	m_HBox1.set_border_width(5);

	pVBox->pack_start (m_HBox2, Gtk::PACK_SHRINK);
	m_HBox2.set_border_width(5);

	add_button (Gtk::Stock::CLOSE, Gtk::RESPONSE_CLOSE);

	show_all_children();

};

Formats::~Formats()
{
}

