#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtkmm/main.h>
#include <glibmm/optionentry.h>
#include <glibmm/optiongroup.h>
#include <glibmm/optioncontext.h>
#include <gtk/gtk.h>
#include <iostream>

#include "check.hpp"
#include "mainwindow.hpp"

const char	*program_name;
bool verbose = false;
std::string sep = "\n------------------------------------------------------------------------\n";

Glib::OptionEntry make_entry (const Glib::ustring& long_name, const gchar& short_name, const Glib::ustring& description)
{

	Glib::OptionEntry m_OptionEntry;

	if (!long_name.empty())
		m_OptionEntry.set_long_name (long_name);
	if (short_name)
		m_OptionEntry.set_short_name (short_name);
	if (!description.empty())
		m_OptionEntry.set_description (description);

	return m_OptionEntry;

}

int main (int argc, char *argv[])
{

	bool version = false;
	bool ffversion = false;

	Glib::OptionGroup m_OptionGroup ("main", "main options", "help for main options");
	m_OptionGroup.add_entry (make_entry ("verbose", 'v', "Print verbose messages."), verbose);
	m_OptionGroup.add_entry (make_entry ("version", 'V', "Print programs version."), version);
	m_OptionGroup.add_entry (make_entry ("ffversion", 'f', "Print ffmpeg version."), ffversion);

	Glib::OptionContext m_OptionContext;
	m_OptionContext.set_main_group (m_OptionGroup);

	Gtk::Main kit(argc, argv, m_OptionContext);

	try
	{
		m_OptionContext.parse (argc, argv);
	}
	catch (const Glib::OptionError& error)
	{
		std::cout << sep << error.what() << std::endl;
	}

	if (version)
	{
		std::cout << VERSION <<std::endl;
		exit (0);
	}

	if (ffversion)
	{
		system ("ffmpeg -version");
		exit (0);
	}

	if(!Glib::thread_supported())
		Glib::thread_init();

	get_formats();
	get_codecs();

	if( check_ffmpeg() )
	{
		MainWindow mainwindow;
		Gtk::Main::run(mainwindow);
	}

	std::cout << std::endl;

	return 0;

}
