#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "mainwindow.hpp"
#include "xmlreader.hpp"
#include "converter.hpp"
#include "info.hpp"
#include "avinfo.hpp"
#include "check.hpp"
#include "formatsdialog.hpp"
#include "compose.hpp"
#include "ucompose.hpp"
#include "time.hpp"
#include "string.hpp"
#include "about.hpp"

#include <gtkmm/filechooserdialog.h>
#include <gtkmm/messagedialog.h>
#include <sys/wait.h>
#include <signal.h>
#include <iostream>

extern std::string sep; // from main.cpp
extern bool converting;
extern char info[200];  // from info.cpp

MainWindow::MainWindow()
:	home(Glib::get_home_dir()), current_open_path(home), current_save_path(home),
	m_Frame_Input("Input:"), m_Frame_Output("Output:"), m_Frame_Info("File info:"),
	m_RadioButton1("User's Home directory: " + home), m_RadioButton2("Other:"), m_Button_Choose("Choose"),
	timeout_value(500), current_id(1)
{
	set_position (Gtk::WIN_POS_CENTER);
	set_title(PACKAGE);
	set_default_size(600, 500);
	add (m_VBox_Main);

	// Menus & ToolBar
	// Create actions for menus and toolbar:
	m_refActionGroup = Gtk::ActionGroup::create();

	// File menu
	m_refActionGroup->add (Gtk::Action::create ("FileMenu", "_File"));
	m_refActionGroup->add (Gtk::Action::create ("FileAdd", Gtk::Stock::ADD),
		sigc::mem_fun(*this, &MainWindow::on_menu_file_add));
	m_refActionGroup->add (Gtk::Action::create ("FileQuit", Gtk::Stock::QUIT),
		sigc::mem_fun(*this, &MainWindow::on_menu_file_quit));

	// Edit menu
	m_refActionGroup->add (Gtk::Action::create ("EditMenu", "_Edit"));
	m_refActionGroup->add (Gtk::Action::create ("EditConvert", Gtk::Stock::CONVERT),
		sigc::mem_fun(*this, &MainWindow::on_menu_edit_convert));
	m_refActionGroup->add (Gtk::Action::create ("EditStop", Gtk::Stock::STOP),
		sigc::mem_fun(*this, &MainWindow::on_menu_edit_stop));
	m_refActionGroup->add (Gtk::Action::create ("EditRemove", Gtk::Stock::REMOVE),
		sigc::mem_fun(*this, &MainWindow::on_menu_edit_remove));
	m_refActionGroup->add (Gtk::Action::create ("EditClear", Gtk::Stock::CLEAR),
		sigc::mem_fun(*this, &MainWindow::on_menu_edit_clear));

	// View Menu
	m_refActionGroup->add (Gtk::Action::create ("ViewMenu", "_View"));
	m_refActionGroup->add (Gtk::Action::create ("ViewFormats", "Supported _formats", "Show the list of supported formats and codecs"),
		Gtk::AccelKey ('f', Gdk::CONTROL_MASK, "<ui>/MenuBar/ViewMenu/ViewFormats"),
		sigc::mem_fun(*this, &MainWindow::on_menu_view_formats));

	// Help menu
	m_refActionGroup->add  (Gtk::Action::create ("HelpMenu", "_Help"));
	m_refActionGroup->add  (Gtk::Action::create ("HelpAbout", Gtk::Stock::ABOUT),
		sigc::mem_fun(*this, &MainWindow::on_menu_help_about));

	m_refUIManager = Gtk::UIManager::create();
	m_refUIManager->insert_action_group(m_refActionGroup);

	add_accel_group (m_refUIManager->get_accel_group());

	//Layout the actions in a menubar and toolbar:
	Glib::ustring ui_info =
		"<ui>"
		"	<menubar name='MenuBar'>"
		"		<menu action='FileMenu'>"
		"			<menuitem action='FileAdd'/>"
		"			<separator/>"
		"			<menuitem action='FileQuit'/>"
		"		</menu>"
		"		<menu action='EditMenu'>"
		"			<menuitem action='EditRemove'/>"
		"			<menuitem action='EditClear'/>"
		"		</menu>"
		"		<menu action='ViewMenu'>"
		"			<menuitem action='ViewFormats'/>"
		"		</menu>"
		"		<menu action='HelpMenu'>"
		"			<menuitem action='HelpAbout'/>"
		"		</menu>"
		"	</menubar>"
		"	<toolbar name='ToolBar'>"
		"		<toolitem action='FileAdd'/>"
		"		<toolitem action='EditRemove'/>"
		"		<toolitem action='EditClear'/>"
		"		<separator/>"
		"		<toolitem action='EditConvert'/>"
		"		<toolitem action='EditStop'/>"
		"	</toolbar>"
		"</ui>";

	#ifdef GLIBMM_EXCEPTIONS_ENABLED
	try
	{	  
		m_refUIManager->add_ui_from_string(ui_info);
	}
	catch(const Glib::Error& ex)
	{
		std::cerr << sep << "building menus failed: " <<  ex.what() << std::endl;
	}
	#else
	std::auto_ptr<Glib::Error> ex;
	m_refUIManager->add_ui_from_string(ui_info, ex);
	if(ex.get())
	{ 
		std::cerr << sep << "building menus failed: " <<  ex->what() << std::endl;
	}
	#endif //GLIBMM_EXCEPTIONS_ENABLED

	//Get the menubar and toolbar widgets, and add them to a container widget:
	Gtk::Widget* pMenubar = m_refUIManager->get_widget("/MenuBar");
	if(pMenubar)
		m_VBox_Main.pack_start (*pMenubar, Gtk::PACK_SHRINK);

	Gtk::Widget* pToolbar = m_refUIManager->get_widget("/ToolBar") ;
	if(pToolbar)
		m_VBox_Main.pack_start (*pToolbar, Gtk::PACK_SHRINK);


	m_VBox_Main.pack_start (m_VBox_Middle, Gtk::PACK_EXPAND_WIDGET, 3);
	m_VBox_Middle.set_border_width (2);
	m_VBox_Middle.pack_start (m_HPaned);

		m_HPaned.set_position (400);
		m_HPaned.pack1 (m_VBox_Left);

			m_VBox_Left.set_spacing (5);
			m_VBox_Left.pack_start (m_Frame_TreeView);
			m_Frame_TreeView.set_shadow_type (Gtk::SHADOW_IN);
			m_Frame_TreeView.add (m_ScrolledWindow);

			//Tree
			//Add the TreeView, inside a ScrolledWindow:
			m_ScrolledWindow.add (m_TreeView);

			//Only show the scrollbars when they are necessary:
			m_ScrolledWindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

			//Create the Tree model for Tree:
			m_Tree_refTreeModel = Gtk::ListStore::create (m_Tree_Columns);
			m_TreeView.set_model (m_Tree_refTreeModel);
			m_TreeView.set_grid_lines (Gtk::TREE_VIEW_GRID_LINES_VERTICAL);

			//Add the TreeView's view columns:
			m_TreeView.append_column ("#", m_Tree_Columns.m_col_id);
			m_TreeView.append_column ("Support", m_Tree_Columns.m_col_supported);
			m_TreeView.append_column ("Duration", m_Tree_Columns.m_col_duration);

			Gtk::CellRendererText* pRendererFilePath = Gtk::manage (new Gtk::CellRendererText());
			pRendererFilePath->property_ellipsize() = Pango::ELLIPSIZE_MIDDLE;

			int cols_count = m_TreeView.append_column ("File path", *pRendererFilePath);
			Gtk::TreeViewColumn* pColumn = m_TreeView.get_column(cols_count-1);
			if(pColumn)
			{
				pColumn->add_attribute(pRendererFilePath->property_text(), m_Tree_Columns.m_col_path);
				pColumn->set_resizable();
			}

			Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_TreeView.get_selection();
			refTreeSelection->signal_changed().connect (sigc::mem_fun(*this, &MainWindow::on_treeselection_changed));

			// Notebook
			m_VBox_Left.pack_start (m_Notebook, Gtk::PACK_SHRINK);
				m_Notebook.append_page (m_VBox_Format, "Options");
					m_VBox_Format.set_border_width(10);
					m_VBox_Format.set_spacing(5);
					m_VBox_Format.pack_start (m_HBox_Format, Gtk::PACK_SHRINK);
						m_HBox_Format.set_spacing(5);

						m_Label_Format.set_markup ("<b>Format:</b>");
						m_HBox_Format.pack_start (m_Label_Format, Gtk::PACK_SHRINK);
				

						// ComboBox
						// Create the Tree model for ComboBox:
						m_Combo_refTreeModel = Gtk::TreeStore::create(m_Combo_Columns);
						m_Combo_Format.set_model(m_Combo_refTreeModel);

						Gtk::CellRendererText* pRendererName = Gtk::manage (new Gtk::CellRendererText());
						Gtk::CellRendererText* pRendererExt = Gtk::manage (new Gtk::CellRendererText());
						m_Combo_Format.pack_start (*pRendererName);
						m_Combo_Format.add_attribute (pRendererName->property_text(), m_Combo_Columns.m_col_name);
						m_Combo_Format.add_attribute (pRendererName->property_sensitive(), m_Combo_Columns.m_col_sense);
						m_Combo_Format.pack_start (*pRendererExt);
						m_Combo_Format.add_attribute (pRendererExt->property_text(), m_Combo_Columns.m_col_extension);
						m_Combo_Format.add_attribute (pRendererExt->property_sensitive(), m_Combo_Columns.m_col_sense);
						pRendererExt->property_xalign() = 1.0;
						m_Combo_Format.set_size_request (300);
						m_Combo_Format.signal_changed().connect (sigc::mem_fun (*this, &MainWindow::on_combo_changed));

						// Read formats data from xml files
						xmlreader (&m_Combo_Columns, m_Combo_refTreeModel);

						m_HBox_Format.pack_start (m_Combo_Format, Gtk::PACK_SHRINK, 5);

					m_VBox_Format.pack_start (m_Label_Format_Info, Gtk::PACK_SHRINK);

				m_Notebook.append_page (m_VBox_SaveTo, "Save to");

					m_VBox_SaveTo.set_border_width(10);
					m_VBox_SaveTo.set_spacing(5);

					// Put radio button 2 in the same group as 1:
					Gtk::RadioButton::Group group = m_RadioButton1.get_group();
					m_RadioButton2.set_group(group);
					m_RadioButton2.signal_toggled().connect (sigc::mem_fun (*this, &MainWindow::on_radiobutton2_change));

					m_VBox_SaveTo.pack_start (m_HBox_rb1, Gtk::PACK_SHRINK);
						m_HBox_rb1.pack_start (m_RadioButton1, Gtk::PACK_SHRINK);

					m_VBox_SaveTo.pack_start (m_HBox_rb2, Gtk::PACK_SHRINK);

						m_HBox_rb2.pack_start (m_RadioButton2, Gtk::PACK_SHRINK);
						m_HBox_rb2.pack_start (m_Entry_rb2);

							m_Entry_rb2.set_sensitive (false);
							m_Entry_rb2.set_text (home);

						m_HBox_rb2.pack_start (m_Button_Choose, Gtk::PACK_SHRINK, 5);

						m_Tooltips.set_tip (m_Button_Choose, "Choose a directory");

							m_Button_Choose.set_sensitive (false);
							m_Button_Choose.set_size_request (100);
							m_Button_Choose.signal_clicked().connect (sigc::mem_fun(*this, &MainWindow::on_button_choose));

		m_HPaned.pack2 (m_VBox_Right, Gtk::FILL);

			m_VBox_Right.set_spacing (5);
			m_VBox_Right.pack_start (m_Frame_Info, Gtk::PACK_SHRINK);
				m_Frame_Info.add (m_VBox_Info);
				m_Frame_Info.set_shadow_type (Gtk::SHADOW_IN);
					m_VBox_Info.set_border_width(5);
					m_VBox_Info.set_spacing (10);

					m_VBox_Info.pack_start (m_HBox_Info, Gtk::PACK_SHRINK);
						m_HBox_Info.set_spacing (5);
						m_HBox_Info.pack_start (m_Label_Info_Labels, Gtk::PACK_SHRINK);
							m_Label_Info_Labels.set_justify (Gtk::JUSTIFY_RIGHT);
							m_Label_Info_Labels.set_markup ("<b>Frame:\nfps:\nQuantizer:\nBitrate:\nSize:\n\nPosition:\nTime used:\nEstimated:\nRemaining:</b>");

						m_HBox_Info.pack_start (m_Label_Info_Infos, Gtk::PACK_SHRINK);

					m_VBox_Info.pack_start (m_ProgressBar, Gtk::PACK_SHRINK);
						m_ProgressBar.set_text ("0%");

			m_VBox_Right.pack_start (m_Frame_Input, Gtk::PACK_SHRINK);
				m_Frame_Input.add (m_VBox_Input);
				m_Frame_Input.set_shadow_type (Gtk::SHADOW_IN);
					m_VBox_Input.set_border_width(5);
					m_VBox_Input.set_spacing(5);

					m_VBox_Input.pack_start (m_VBox_Input_Video, Gtk::PACK_SHRINK);
					m_VBox_Input_Video.pack_start (m_HBox_Input_Video_Title, Gtk::PACK_SHRINK);

						m_Image_Input_Video.set_from_icon_name ("video-x-generic", Gtk::ICON_SIZE_DND);
						m_HBox_Input_Video_Title.pack_start (m_Image_Input_Video, Gtk::PACK_SHRINK);
						m_HBox_Input_Video_Title.pack_start (m_Label_Input_Video_Title, Gtk::PACK_SHRINK);
							m_Label_Input_Video_Title.set_markup ("<b>Video</b>");

					m_VBox_Input_Video.pack_start (m_HBox_Input_Video, Gtk::PACK_SHRINK);
						m_HBox_Input_Video.set_spacing(5);

						m_Label_Input_Video_Labels.set_markup ("<b>Size:\nAspect:\nCodec:\nBitrate:\n  Framerate:</b>");
						m_Label_Input_Video_Labels.set_justify (Gtk::JUSTIFY_RIGHT);
						m_HBox_Input_Video.pack_start (m_Label_Input_Video_Labels, Gtk::PACK_SHRINK);

						m_HBox_Input_Video.pack_start (m_Label_Input_Video_Infos, Gtk::PACK_SHRINK);


					m_VBox_Input.pack_start (m_VBox_Input_Audio, Gtk::PACK_SHRINK);
					m_VBox_Input_Audio.pack_start (m_HBox_Input_Audio_Title, Gtk::PACK_SHRINK);
						
						m_Image_Input_Audio.set_from_icon_name ("audio-x-generic", Gtk::ICON_SIZE_DND);
						m_HBox_Input_Audio_Title.pack_start (m_Image_Input_Audio, Gtk::PACK_SHRINK);
						m_HBox_Input_Audio_Title.pack_start (m_Label_Input_Audio_Title, Gtk::PACK_SHRINK);
							m_Label_Input_Audio_Title.set_markup ("<b>Audio</b>");

					m_VBox_Input_Audio.pack_start (m_HBox_Input_Audio, Gtk::PACK_SHRINK);
						m_HBox_Input_Audio.set_spacing(5);

						m_Label_Input_Audio_Labels.set_markup ("<b>Codec:\nBitrate:\nSamplerate:</b>");
						m_Label_Input_Audio_Labels.set_justify (Gtk::JUSTIFY_RIGHT);
						m_HBox_Input_Audio.pack_start (m_Label_Input_Audio_Labels, Gtk::PACK_SHRINK);

						m_HBox_Input_Audio.pack_start (m_Label_Input_Audio_Infos, Gtk::PACK_SHRINK);


			m_VBox_Right.pack_start (m_Frame_Output, Gtk::PACK_SHRINK);
				m_Frame_Output.add (m_VBox_Output);
				m_Frame_Output.set_shadow_type (Gtk::SHADOW_IN);
					m_VBox_Output.set_border_width(5);
					m_VBox_Output.set_spacing(5);

					m_VBox_Output.pack_start (m_VBox_Output_Video, Gtk::PACK_SHRINK);
					m_VBox_Output_Video.pack_start (m_HBox_Output_Video_Title, Gtk::PACK_SHRINK);
						m_HBox_Output_Video_Title.pack_start (m_Image_Output_Video, Gtk::PACK_SHRINK);
							m_Image_Output_Video.set_from_icon_name ("video-x-generic", Gtk::ICON_SIZE_DND);
						m_HBox_Output_Video_Title.pack_start (m_Label_Output_Video_Title, Gtk::PACK_SHRINK);
							m_Label_Output_Video_Title.set_markup ("<b>Video</b>");

					m_VBox_Output_Video.pack_start (m_HBox_Output_Video, Gtk::PACK_SHRINK);
						m_HBox_Output_Video.set_spacing(5);

						m_HBox_Output_Video.pack_start (m_Label_Output_Video_Labels, Gtk::PACK_SHRINK);
							m_Label_Output_Video_Labels.set_markup ("<b>Size:\nAspect:\nCodec:\nBitrate:\n  Framerate:</b>");
							m_Label_Output_Video_Labels.set_justify (Gtk::JUSTIFY_RIGHT);

						m_HBox_Output_Video.pack_start (m_Label_Output_Video_Infos, Gtk::PACK_SHRINK);

					m_VBox_Output.pack_start (m_VBox_Output_Audio, Gtk::PACK_SHRINK);
						m_VBox_Output_Audio.pack_start (m_HBox_Output_Audio_Title, Gtk::PACK_SHRINK);

						m_Image_Output_Audio.set_from_icon_name ("audio-x-generic", Gtk::ICON_SIZE_DND);
						m_HBox_Output_Audio_Title.pack_start (m_Image_Output_Audio, Gtk::PACK_SHRINK);
						m_HBox_Output_Audio_Title.pack_start (m_Label_Output_Audio_Title, Gtk::PACK_SHRINK);
							m_Label_Output_Audio_Title.set_markup ("<b>Audio</b>");

					m_VBox_Output_Audio.pack_start (m_HBox_Output_Audio, Gtk::PACK_SHRINK);
						m_HBox_Output_Audio.set_spacing(5);

						m_HBox_Output_Audio.pack_start (m_Label_Output_Audio_Labels, Gtk::PACK_SHRINK);
							m_Label_Output_Audio_Labels.set_markup ("<b>Codec:\nBitrate:\nSamplerate:</b>");
							m_Label_Output_Audio_Labels.set_justify (Gtk::JUSTIFY_RIGHT);

						m_HBox_Output_Audio.pack_start (m_Label_Output_Audio_Infos, Gtk::PACK_SHRINK);


	m_VBox_Main.pack_start (m_Statusbar, Gtk::PACK_SHRINK);

	show_all_children();
	set_sense (START);

}

MainWindow::~MainWindow()
{
}

void MainWindow::on_menu_file_add()
{

	Gtk::FileChooserDialog dialog ("Select the video file(s) to add", Gtk::FILE_CHOOSER_ACTION_OPEN);
	dialog.set_current_folder (current_open_path);
	dialog.set_select_multiple (true);
	dialog.set_transient_for (*this);

	dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);

	Gtk::FileFilter filter_video;
	filter_video.set_name ("Video files");
	filter_video.add_mime_type ("video/*");
	dialog.add_filter (filter_video);

	Gtk::FileFilter filter_any;
	filter_any.set_name ("All files");
	filter_any.add_pattern ("*");
	dialog.add_filter (filter_any);

	int result = dialog.run();

	switch (result)
	{

		case (Gtk::RESPONSE_OK):
		{
			std::list<std::string> filenames = dialog.get_filenames();
			Gtk::TreeModel::Row row;

			for (std::list<std::string>::iterator iter = filenames.begin(); iter != filenames.end(); ++iter)
			{
				AVInfo tmp_avinfo;
				if(get_avinfo (&tmp_avinfo, *iter) == 0) // if we succeeded to open and get file info
				{
					row = *(m_Tree_refTreeModel->append());

					row[m_Tree_Columns.m_col_path] = *iter;


					if(check_decodec (tmp_avinfo.get_vcodec())	&&	check_decodec (tmp_avinfo.get_acodec()))
						row[m_Tree_Columns.m_col_supported] = true;
					else
						row[m_Tree_Columns.m_col_supported] = false;

					row[m_Tree_Columns.m_col_duration] = get_formated_time (tmp_avinfo.get_duration());

					row[m_Tree_Columns.m_col_avinfo] = tmp_avinfo;

					regenerate_tree_id();
				}

				if (iter == filenames.begin())
				{
					std::string tmp = *iter;
					current_open_path.assign (tmp, 0, tmp.rfind ("/"));
				}
			}

			type_tree_children children = m_Tree_refTreeModel->children();
			if(children.size())
				set_sense (CONTAIN);
			break;
		}

		default:
		{
			break;
		}

	}

}

void MainWindow::on_menu_file_quit()
{

	on_menu_edit_stop();
	hide();

}

void MainWindow::on_menu_edit_remove()
{

	Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_TreeView.get_selection();
	Gtk::TreeModel::iterator iter = refTreeSelection->get_selected();

	if(iter) //If anything is selected
	{
		Gtk::TreeModel::Row row = *iter;
		m_Tree_refTreeModel->erase (row);
		regenerate_tree_id();
		if (m_Tree_refTreeModel->children().empty())
			set_sense (START);
		m_Label_Input_Video_Infos.set_text(" ");
		m_Label_Input_Audio_Infos.set_text(" ");
	}

}

void MainWindow::on_menu_edit_clear()
{

	m_Tree_refTreeModel->clear();
	set_sense (START);
	m_Label_Input_Video_Infos.set_text(" ");
	m_Label_Input_Audio_Infos.set_text(" ");

}

void MainWindow::on_menu_edit_convert()
{

	Gtk::TreeModel::iterator combo_iter = m_Combo_Format.get_active();
	Gtk::TreeModel::Row combo_row;
	if (combo_iter)
	{
		combo_row = *combo_iter;

		type_tree_children tree_children = m_Tree_refTreeModel->children();

		std::list< io_t > io_list;

		for(type_tree_children::iterator tree_iter = tree_children.begin(); tree_iter != tree_children.end(); ++tree_iter)
		{
			Gtk::TreeModel::Row tree_row = *tree_iter;

			bool canceled = false;

			io_t tmpio;

			tmpio.input.path	= tree_row[m_Tree_Columns.m_col_path];

			tmpio.output.reconvert	= combo_row[ m_Combo_Columns.m_col_reconvert];

			tmpio.output.sizew	= combo_row[ m_Combo_Columns.m_col_sizew ];
			tmpio.output.sizeh	= combo_row[ m_Combo_Columns.m_col_sizeh ];
			tmpio.output.aspectw	= combo_row[ m_Combo_Columns.m_col_aspectw ];
			tmpio.output.aspecth	= combo_row[ m_Combo_Columns.m_col_aspecth ];
			tmpio.output.vcodec	= combo_row[ m_Combo_Columns.m_col_vcodec ];
			tmpio.output.vbitrate	= combo_row[ m_Combo_Columns.m_col_vbitrate ];
			tmpio.output.framerate	= combo_row[ m_Combo_Columns.m_col_framerate ];

			tmpio.output.usepix_fmt	= combo_row[ m_Combo_Columns.m_col_usepix_fmt ];
			tmpio.output.pix_fmt	= combo_row[ m_Combo_Columns.m_col_pix_fmt ];

			tmpio.output.acodec	= combo_row[ m_Combo_Columns.m_col_acodec ];
			tmpio.output.abitrate	= combo_row[ m_Combo_Columns.m_col_abitrate ];
			tmpio.output.samplerate	= combo_row[ m_Combo_Columns.m_col_samplerate ];
			tmpio.output.achannels	= combo_row[ m_Combo_Columns.m_col_achannels ];

			tmpio.output.useflags	= combo_row[ m_Combo_Columns.m_col_useflags ];
			tmpio.output.mbd	= combo_row[ m_Combo_Columns.m_col_mbd ];
			tmpio.output.flags	= combo_row[ m_Combo_Columns.m_col_flags ];
			tmpio.output.cmp	= combo_row[ m_Combo_Columns.m_col_cmp ];
			tmpio.output.subcmp	= combo_row[ m_Combo_Columns.m_col_subcmp ];
			tmpio.output.g		= combo_row[ m_Combo_Columns.m_col_g ];

			std::string output_path;
			if (m_RadioButton1.get_active())
			{
				output_path	= home;

			}
			if (m_RadioButton2.get_active())
			{
				output_path	= m_Entry_rb2.get_text();
			}
			output_path.append ("/" + Glib::path_get_basename (tree_row[m_Tree_Columns.m_col_path]));
			tmpio.output.path	= replace_ext (output_path, combo_row[m_Combo_Columns.m_col_extension]);


			if(Glib::file_test (tmpio.output.path, Glib::FILE_TEST_EXISTS))
			{
				Glib::ustring message = String::ucompose ("<b>A File named '%1' already exists. Do you want to replace it?</b>",
										Glib::path_get_basename (tmpio.output.path));
				Gtk::MessageDialog dialog (*this, message, true, Gtk::MESSAGE_QUESTION , Gtk::BUTTONS_YES_NO, true);
				dialog.set_secondary_text (String::ucompose ("The file already exists in '%1'. Replacing it will overwrite it's contents.",
										Glib::path_get_dirname (tmpio.output.path)));
				int result = dialog.run();

				switch(result)
				{
					case(Gtk::RESPONSE_YES):
					{
						tmpio.output.overwrite = true;
						break;
					}
					case(Gtk::RESPONSE_NO):
					{
						tmpio.output.overwrite = false;

						Gtk::FileChooserDialog dialog (*this, "Save as", Gtk::FILE_CHOOSER_ACTION_SAVE);
						dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
						dialog.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_OK);
						dialog.set_do_overwrite_confirmation();
						dialog.set_current_folder(Glib::path_get_dirname (tmpio.output.path));
						std::string current_name = Glib::path_get_basename (tree_row[m_Tree_Columns.m_col_path]);
						current_name.insert (current_name.rfind("."), "[2]");
						dialog.set_current_name (current_name);

						Gtk::FileFilter filter_video;
						filter_video.set_name ("Video files");
						filter_video.add_mime_type ("video/*");
						dialog.add_filter (filter_video);

						Gtk::FileFilter filter_any;
						filter_any.set_name ("All files");
						filter_any.add_pattern ("*");
						dialog.add_filter (filter_any);

						int result = dialog.run();

						switch(result)
						{
							case(Gtk::RESPONSE_OK):
							{
								tmpio.output.path = dialog.get_filename();
								tmpio.output.overwrite = true;
								break;
							}
							case(Gtk::RESPONSE_CANCEL):
							{
								canceled = true;
								break;
							}
						}
						break;
					}
				}
			}

			AVInfo tmp_avinfo	= tree_row[m_Tree_Columns.m_col_avinfo];
			tmpio.input.sizew	= tmp_avinfo.get_width();
			tmpio.input.sizeh	= tmp_avinfo.get_height();
			tmpio.input.aspect	= tmp_avinfo.get_aspect();

			if(!canceled)
				io_list.push_back (tmpio);
		}

		set_sense (CONVERT);

		//Create the timer & start checking for child processes every second
		current_time = 0;
		total_time = 0;
		sigc::slot<bool> my_slot = sigc::mem_fun (*this, &MainWindow::on_timeout);
		sigc::connection conn = Glib::signal_timeout().connect(my_slot, timeout_value);
		m_timer = conn;

		child_pid = convert (io_list);
		converting = true;
		Glib::Thread::create (sigc::ptr_fun (&get_info), false);
	}
	else
	{
		Glib::ustring message = "<b>Select a file format first and then click convert!</b>";
		Gtk::MessageDialog dialog (*this, message, true, Gtk::MESSAGE_WARNING , Gtk::BUTTONS_OK);
		dialog.run();
	}

}

void MainWindow::on_menu_edit_stop()
{

	if (child_pid)
		kill (child_pid, SIGTERM);

	set_sense (STOP);

}

void MainWindow::on_menu_view_formats()
{

	Formats formats;
	formats.run();

}

void MainWindow::on_menu_help_about()
{

	AboutDialog aboutdialog;
	aboutdialog.run();

}

// update Output frame infos when ComboBox is changed, and check for video and audio codecs
void MainWindow::on_combo_changed()
{

	Gtk::TreeModel::iterator iter = m_Combo_Format.get_active();
	if(iter)
	{ 
		Gtk::TreeModel::Row row = *iter;

		//Get the data for the selected row, using our knowledge of the tree model:
		std::string vcodec = row[m_Combo_Columns.m_col_vcodec];
		std::string acodec = row[m_Combo_Columns.m_col_acodec];

		if(!check_encodec (vcodec) || !check_encodec (acodec))
		{

			if(!check_encodec (vcodec) && check_encodec ("lib" + vcodec))
			{
				vcodec = "lib" + vcodec;
				row[m_Combo_Columns.m_col_vcodec] = vcodec;
			}

			if(!check_encodec (acodec) && check_encodec ("lib" + acodec))
			{
				acodec ="lib" + acodec;
				row[m_Combo_Columns.m_col_acodec] = acodec;
				m_Label_Format_Info.set_markup (" ");
			}
			else
			{
				if(acodec == "aac" && check_encodec ("libfaac"))
				{
					acodec = "libfaac";
					row[m_Combo_Columns.m_col_acodec] = acodec;
					m_Label_Format_Info.set_markup (" ");
				}
				else if(acodec == "mp3" && check_encodec ("libmp3lame"))
				{
					acodec = "libmp3lame";
					row[m_Combo_Columns.m_col_acodec] = acodec;
					m_Label_Format_Info.set_markup (" ");
				}
			}

			if(check_encodec (vcodec) && check_encodec (acodec))
				m_Label_Format_Info.set_markup (" ");
			else
				m_Label_Format_Info.set_markup ("<span weight='bold' foreground='red'>The selected format is not supported!</span>");
		}
		else
			m_Label_Format_Info.set_markup (" ");

		std::string video = String::compose ("%1x%2\n%3:%4\n%5\n%6\n%7",
							std::string (row[m_Combo_Columns.m_col_sizew]),
							std::string (row[m_Combo_Columns.m_col_sizeh]),
							std::string (row[m_Combo_Columns.m_col_aspectw]),
							std::string (row[m_Combo_Columns.m_col_aspecth]),
							std::string (row[m_Combo_Columns.m_col_vcodec]),
							std::string (row[m_Combo_Columns.m_col_vbitrate]),
							std::string (row[m_Combo_Columns.m_col_framerate]));
		m_Label_Output_Video_Infos.set_text (video);

		std::string audio = String::compose ("%1\n%2\n%3",
							std::string (row[m_Combo_Columns.m_col_acodec]),
							std::string (row[m_Combo_Columns.m_col_abitrate]),
							std::string (row[m_Combo_Columns.m_col_samplerate]));
		m_Label_Output_Audio_Infos.set_text (audio);
	}

}

void MainWindow::on_radiobutton2_change()
{

	if (m_RadioButton2.get_active())
	{
		m_Entry_rb2.set_sensitive(true);
		m_Button_Choose.set_sensitive(true);
	}
	else
	{
		m_Entry_rb2.set_sensitive(false);
		m_Button_Choose.set_sensitive(false);
	}

}

void MainWindow::on_button_choose()
{

	Gtk::FileChooserDialog dialog ("Select a folder to save video file(s)", Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER);
	dialog.set_current_folder (current_save_path);
	dialog.set_transient_for (*this);

	dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);

	int result = dialog.run();

	switch (result)
	{

		case (Gtk::RESPONSE_OK):
		{
			m_Entry_rb2.set_text (dialog.get_filename());
			current_save_path = dialog.get_filename();
			break;
		}

		default:
		{
			break;
		}

	}

}

// update Input frame infos when a row is selected
void MainWindow::on_treeselection_changed()
{

	Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_TreeView.get_selection();
	Gtk::TreeModel::iterator iter = refTreeSelection->get_selected();

	if(iter) //If anything is selected
	{
		Gtk::TreeModel::Row row = *iter;
		AVInfo info = row[m_Tree_Columns.m_col_avinfo];

		std::ostringstream video, audio;


		video	<< info.get_width() << "x" << info.get_height() << "\n";

		unsigned int aspect = info.get_aspect()*10;
		switch (aspect)
		{
			case 13:
				video	<< "4:3";
				break;
			case 17:
				video	<< "16:9";
				break;
			default:
				video	<< info.get_aspect();
		}

		video	<< "\n" << info.get_vcodec() << "\n";

		if(info.get_vbitrate())
		{
			if(info.get_vbitrate()>100000)
				video	<< info.get_vbitrate()/1000 << "K\n";
			else
				video	<< info.get_vbitrate() << "\n";
		}
		else
			video	<< "unknown\n";

		video	<< info.get_framerate();

		m_Label_Input_Video_Infos.set_text (video.str());



		audio	<< info.get_acodec() << "\n";

		if(info.get_abitrate())
		{
			if(info.get_abitrate()>100000)
				audio	<< info.get_abitrate()/1000 << "K\n";
			else
				audio	<< info.get_abitrate() << "\n";
		}
		else
			audio	<< "unknown\n";

		audio	<< info.get_samplerate();

		m_Label_Input_Audio_Infos.set_text (audio.str());
	}

}

void MainWindow::regenerate_tree_id()
{

	type_tree_children tree_children = m_Tree_refTreeModel->children();

	int i = 1;
	for(type_tree_children::iterator tree_iter = tree_children.begin(); tree_iter != tree_children.end(); ++tree_iter)
	{
		Gtk::TreeModel::Row tree_row = *tree_iter;

		tree_row[m_Tree_Columns.m_col_id] = i;

		i++;
	}

}

void MainWindow::set_sense (int STATUS)
{

	switch (STATUS)
	{
		case (START):
		{
			m_Frame_Info.hide();
			m_refUIManager->get_widget("/MenuBar/EditMenu/EditRemove")->set_sensitive(false);
			m_refUIManager->get_widget("/MenuBar/EditMenu/EditClear")->set_sensitive(false);
			m_refUIManager->get_widget("/ToolBar/EditRemove")->set_sensitive(false);
			m_refUIManager->get_widget("/ToolBar/EditClear")->set_sensitive(false);
			m_refUIManager->get_widget("/ToolBar/EditConvert")->set_sensitive(false);
			m_refUIManager->get_widget("/ToolBar/EditStop")->set_sensitive(false);
			break;
		}
		case (CONTAIN):
		{
			m_refUIManager->get_widget("/MenuBar/EditMenu/EditRemove")->set_sensitive(true);
			m_refUIManager->get_widget("/MenuBar/EditMenu/EditClear")->set_sensitive(true);
			m_refUIManager->get_widget("/ToolBar/EditRemove")->set_sensitive(true);
			m_refUIManager->get_widget("/ToolBar/EditClear")->set_sensitive(true);
			m_refUIManager->get_widget("/ToolBar/EditConvert")->set_sensitive(true);
			break;
		}
		case (CONVERT):
		{
			m_Frame_Input.hide();
			m_Frame_Info.show();
			m_refUIManager->get_widget("/MenuBar/FileMenu/FileAdd")->set_sensitive(false);
			m_refUIManager->get_widget("/MenuBar/EditMenu")->set_sensitive(false);
			m_refUIManager->get_widget("/ToolBar/FileAdd")->set_sensitive(false);
			m_refUIManager->get_widget("/ToolBar/EditRemove")->set_sensitive(false);
			m_refUIManager->get_widget("/ToolBar/EditClear")->set_sensitive(false);
			m_refUIManager->get_widget("/ToolBar/EditConvert")->set_sensitive(false);
			m_refUIManager->get_widget("/ToolBar/EditStop")->set_sensitive(true);
			m_VBox_Left.set_sensitive(false);
			break;
		}
		case (STOP):
		{
			current_id = 1;
			m_Frame_Info.hide();
			m_Label_Info_Infos.set_text (" ");
			m_ProgressBar.set_fraction (0);
			m_ProgressBar.set_text ("0%");
			m_Frame_Input.show();
			m_Statusbar.pop();
			m_refUIManager->get_widget("/MenuBar/FileMenu/FileAdd")->set_sensitive(true);
			m_refUIManager->get_widget("/MenuBar/EditMenu")->set_sensitive(true);
			m_refUIManager->get_widget("/ToolBar/FileAdd")->set_sensitive(true);
			m_refUIManager->get_widget("/ToolBar/EditRemove")->set_sensitive(true);
			m_refUIManager->get_widget("/ToolBar/EditClear")->set_sensitive(true);
			m_refUIManager->get_widget("/ToolBar/EditConvert")->set_sensitive(true);
			m_refUIManager->get_widget("/ToolBar/EditStop")->set_sensitive(false);
			m_VBox_Left.set_sensitive(true);
		}
	}
}

// if user clicks close on window's title bar
bool MainWindow::on_delete_event (GdkEventAny *event)
{

	on_menu_file_quit();
	return true;

}

bool MainWindow::on_timeout()
{

	// check the status of our child
	int status = waitpid (child_pid, &child_status, WNOHANG);

	type_tree_children tree_children = m_Tree_refTreeModel->children();

	// if it's running
	if (status == 0)
	{
		int id;
		char tmp_char[20];
		int input_duration = 0;
		float output_duration = 0;
		std::string tmp_string, string(info), label_text;

		id = atoi (get_value("id", string).c_str());

		total_time += 1;
		if (total_time % 2 == 0)
			current_time += 1;

		// if converting the next file reset current time and change the id
		if(id > current_id)
		{
			current_time = 0;
			current_id = id;
		}

		// if info contains convert stats use them
		if(string.find ("frame") != std::string::npos)
		{
			int i;
			type_tree_children::iterator iter = tree_children.begin();
			for(i=id; i>1; i--)
				iter++;

			if(iter)
			{
				Gtk::TreeModel::Row row = *iter;

				AVInfo tmp_info = row[m_Tree_Columns.m_col_avinfo];
				input_duration = tmp_info.get_duration();
				output_duration = atof (get_value ("time", string).c_str());

				int percent = 100 * output_duration / input_duration;
				m_ProgressBar.set_text (String::compose ("%1%%", percent));
				m_ProgressBar.set_fraction (output_duration / input_duration);
			}

			label_text = String::compose ("%1\n%2\n%3\n%4\n%5\n\n%6\n%7\n%8\n%9",
							get_value ("frame", string),
							get_value ("fps", string),
							get_value ("q", string),
							get_value ("bitrate", string),
							get_value ("size", string),
							get_formated_time (output_duration),
							get_formated_time (current_time),
							get_formated_time (input_duration * current_time / output_duration),
							get_formated_time (input_duration * current_time / output_duration - current_time));

			m_Label_Info_Infos.set_text (label_text);
		}
		// convert stats are not included in info yet
		else
		{
			m_Label_Info_Infos.set_text (" ");
			m_ProgressBar.set_fraction (0);
			m_ProgressBar.set_text ("0%");
		}

		std::string tmp_status = String::compose ("Converting: %1/%2  Total time used: %3  Filename: %4",
								current_id,
								tree_children.size(),
								get_formated_time (total_time/2),
								get_value ("filename", string));
		m_Statusbar.pop(); // clear status bar
		m_Statusbar.push (tmp_status); // set new text

		return true; // continue running the timer
	}
	// if it's not running
	else
	{
		converting = false;
		set_sense (STOP);
		return false; // stop running the timer
	}


}

