#ifndef MAIN_WINDOW_H
#define MAIN_WINDOW_H

#include <gtkmm/window.h>
#include <gtkmm/paned.h>
#include <gtkmm/frame.h>
#include <gtkmm/label.h>
#include <gtkmm/entry.h>
#include <gtkmm/notebook.h>
#include <gtkmm/radiobutton.h>
#include <gtkmm/button.h>
#include <gtkmm/image.h>
#include <gtkmm/stock.h>
#include <gtkmm/combobox.h>
#include <gtkmm/uimanager.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/statusbar.h>
#include <gtkmm/progressbar.h>
#include <gtkmm/tooltips.h>
#include <sys/types.h>

#include "tree_model.hpp"
#include "combo_model.hpp"


class MainWindow : public Gtk::Window
{

public:
	MainWindow();
	virtual ~MainWindow();

protected:
	//Signals
	virtual void on_menu_file_add();
	virtual void on_menu_file_quit();
	virtual void on_menu_edit_convert();
	virtual void on_menu_edit_stop();
	virtual void on_menu_edit_remove();
	virtual void on_menu_edit_clear();
	virtual void on_menu_view_formats();
	virtual void on_menu_help_about();

	virtual void on_combo_changed();
	virtual void on_radiobutton2_change();
	virtual void on_button_choose();
	virtual void on_treeselection_changed();
	virtual void regenerate_tree_id();

	virtual void set_sense( int STATUS );

	virtual bool on_delete_event(GdkEventAny *event);

	bool on_timeout();

	Glib::ustring home, current_open_path, current_save_path;

	//Widgets
	Gtk::VBox	m_VBox_Main, m_VBox_Middle, m_VBox_Left, m_VBox_Right,
			m_VBox_Input, m_VBox_Output,
			m_VBox_Input_Video, m_VBox_Input_Video_Right,
			m_VBox_Input_Audio,
			m_VBox_Output_Video, m_VBox_Output_Audio,
			m_VBox_Info,
			m_VBox_Format, m_VBox_SaveTo;

	Gtk::HBox	m_HBox_Input_Video_Title, m_HBox_Input_Video,
			m_HBox_Input_Audio_Title, m_HBox_Input_Audio,
			m_HBox_Output_Video_Title, m_HBox_Output_Video,
			m_HBox_Output_Audio_Title, m_HBox_Output_Audio,
			m_HBox_Info,
			m_HBox_Format,
			m_HBox_rb1, m_HBox_rb2;

	Gtk::HPaned	m_HPaned;

	Gtk::Frame	m_Frame_TreeView, m_Frame_Input, m_Frame_Output, m_Frame_Info;

	Gtk::Image	m_Image_Input_Video, m_Image_Input_Audio,
			m_Image_Output_Video, m_Image_Output_Audio;

	Gtk::Notebook	m_Notebook;

	Gtk::Label	m_Label_Input_Video_Title, m_Label_Input_Audio_Title,
			m_Label_Output_Video_Title, m_Label_Output_Audio_Title,
			m_Label_Input_Video_Labels, m_Label_Input_Video_Infos,
			m_Label_Input_Audio_Labels, m_Label_Input_Audio_Infos,
			m_Label_Output_Video_Labels, m_Label_Output_Video_Infos,
			m_Label_Output_Audio_Labels, m_Label_Output_Audio_Infos,
			m_Label_Info_Labels, m_Label_Info_Infos,
			m_Label_Format, m_Label_Format_Info;

	Gtk::Entry		m_Entry_rb2;

	Gtk::RadioButton	m_RadioButton1, m_RadioButton2;
	Gtk::Button		m_Button_Choose;

	Gtk::Tooltips		m_Tooltips;
	Gtk::Statusbar		m_Statusbar;
	Gtk::ProgressBar	m_ProgressBar;

	Tree_ModelColumns	m_Tree_Columns;
	Gtk::ScrolledWindow	m_ScrolledWindow;
	Gtk::TreeView		m_TreeView;
	typedef Gtk::TreeModel::Children	type_tree_children; //minimise code length
	Glib::RefPtr<Gtk::ListStore>		m_Tree_refTreeModel;

	Combo_ModelColumns	m_Combo_Columns;
	Glib::RefPtr<Gtk::TreeStore>		m_Combo_refTreeModel;
	Gtk::ComboBox		m_Combo_Format;

	//UIManager & ActionGroup for menus and the toolbar
	Glib::RefPtr <Gtk::UIManager>	m_refUIManager;
	Glib::RefPtr <Gtk::ActionGroup>	m_refActionGroup;

	//Application status
	enum
	{
		START, //Startup
		CONTAIN,//Treeview contains some files
		CONVERT,//Converting
		STOP //Encoding is stoped
	};

	//Child process' pid & status
	pid_t child_pid;
	int child_status;

	// current convert ids
	unsigned int current_id;

	//Timer
	unsigned int total_time, current_time, h, m, s;
	const int timeout_value;
	sigc::connection m_timer;
};

#endif
