#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "xmlreader.hpp"

#include <libxml++/libxml++.h>
#include <libxml++/parsers/textreader.h>
#include <iostream>

extern bool verbose;
extern std::string sep;

// this function read data from xml files and appends the to the ComboBox
int xmlreader( Combo_ModelColumns *m_Columns, Glib::RefPtr<Gtk::TreeStore> m_refTreeModel )
{

	//Read the System wide formats.xml
	try
	{
		Gtk::TreeModel::Row row, childrow;
		xmlpp::Node* current_node;
		xmlpp::TextReader reader( DATASDIR "formats.xml");

		while(reader.read())
		{
			if (reader.get_name() == "device" && reader.has_attributes() && reader.get_current_node() != current_node )
			{
				current_node = reader.get_current_node();

				row = *(m_refTreeModel->append());

				if (verbose)
					std::cout << "Adding Device: " << reader.get_attribute("name") << std::endl;

				row[m_Columns->m_col_sense]		= false;
				row[m_Columns->m_col_name]		= reader.get_attribute("name");
			}
			else if (reader.get_name() == "format" && reader.has_attributes())
			{
				childrow = *(m_refTreeModel->append(row.children()));

				if (verbose)
					std::cout << "\tAdding Format: " << reader.get_attribute("name") << std::endl;

				childrow[m_Columns->m_col_sense]	= true;
				childrow[m_Columns->m_col_name]		= reader.get_attribute("name");

				if(reader.get_attribute("reconvert") == "y")
					childrow[m_Columns->m_col_reconvert]	= true;

				childrow[m_Columns->m_col_sizew]	= reader.get_attribute("sizew");
				childrow[m_Columns->m_col_sizeh]	= reader.get_attribute("sizeh");
				childrow[m_Columns->m_col_aspectw]	= reader.get_attribute("aspectw");
				childrow[m_Columns->m_col_aspecth]	= reader.get_attribute("aspecth");
				childrow[m_Columns->m_col_vcodec]	= reader.get_attribute("vcodec");
				childrow[m_Columns->m_col_vbitrate]	= reader.get_attribute("vbitrate");
				childrow[m_Columns->m_col_framerate]	= reader.get_attribute("framerate");

				if(reader.get_attribute("usepix_fmt") == "y")
				{
					childrow[m_Columns->m_col_usepix_fmt]	= true;
					childrow[m_Columns->m_col_pix_fmt]	= reader.get_attribute("pix_fmt");
				}

				childrow[m_Columns->m_col_acodec]	= reader.get_attribute("acodec");
				childrow[m_Columns->m_col_abitrate]	= reader.get_attribute("abitrate");
				childrow[m_Columns->m_col_samplerate]	= reader.get_attribute("samplerate");
				childrow[m_Columns->m_col_achannels]	= reader.get_attribute("achannels");

				childrow[m_Columns->m_col_extension]	= reader.get_attribute("extension");

				if(reader.get_attribute("useflags") == "y")
				{
					childrow[m_Columns->m_col_useflags]	= true;
					childrow[m_Columns->m_col_mbd]		= reader.get_attribute("mbd");
					childrow[m_Columns->m_col_flags]	= reader.get_attribute("flags");
					childrow[m_Columns->m_col_cmp]		= reader.get_attribute("cmp");
					childrow[m_Columns->m_col_subcmp]	= reader.get_attribute("subcmp");
					childrow[m_Columns->m_col_g]		= reader.get_attribute("g");
				}
			}
		}
	}
	catch(const std::exception& ex)
	{
		if(verbose)
			std::cout << sep << "Exception caught: " << ex.what() << std::endl;
	}

	//Read the formats file from users Home dir
	Glib::ustring user_formats (Glib::get_home_dir());
	user_formats += "/.movic.xml";

	try
	{
		Gtk::TreeModel::Row row, childrow;
		xmlpp::Node* current_node;
		xmlpp::TextReader reader( user_formats );

		while(reader.read())
		{
			if (reader.get_name() == "device" && reader.has_attributes() && reader.get_current_node() != current_node )
			{
				current_node = reader.get_current_node();

				row = *(m_refTreeModel->append());

				if (verbose)
					std::cout << "Adding Device: " << reader.get_attribute("name") << std::endl;

				row[m_Columns->m_col_sense]		= false;
				row[m_Columns->m_col_name]		= reader.get_attribute("name") + "*";
			}
			else if (reader.get_name() == "format" && reader.has_attributes())
			{
				childrow = *(m_refTreeModel->append(row.children()));

				if (verbose)
					std::cout << "\tAdding Format: " << reader.get_attribute("name") << std::endl;

				childrow[m_Columns->m_col_sense]	= true;
				childrow[m_Columns->m_col_name]		= reader.get_attribute("name");

				if(reader.get_attribute("reconvert") == "y")
					childrow[m_Columns->m_col_reconvert]	= true;

				childrow[m_Columns->m_col_sizew]	= reader.get_attribute("sizew");
				childrow[m_Columns->m_col_sizeh]	= reader.get_attribute("sizeh");
				childrow[m_Columns->m_col_aspectw]	= reader.get_attribute("aspectw");
				childrow[m_Columns->m_col_aspecth]	= reader.get_attribute("aspecth");
				childrow[m_Columns->m_col_vcodec]	= reader.get_attribute("vcodec");
				childrow[m_Columns->m_col_vbitrate]	= reader.get_attribute("vbitrate");
				childrow[m_Columns->m_col_framerate]	= reader.get_attribute("framerate");

				if(reader.get_attribute("usepix_fmt") == "y")
				{
					childrow[m_Columns->m_col_usepix_fmt]	= true;
					childrow[m_Columns->m_col_pix_fmt]	= reader.get_attribute("pix_fmt");
				}

				childrow[m_Columns->m_col_acodec]	= reader.get_attribute("acodec");
				childrow[m_Columns->m_col_abitrate]	= reader.get_attribute("abitrate");
				childrow[m_Columns->m_col_samplerate]	= reader.get_attribute("samplerate");
				childrow[m_Columns->m_col_achannels]	= reader.get_attribute("achannels");

				childrow[m_Columns->m_col_extension]	= reader.get_attribute("extension");

				if(reader.get_attribute("useflags") == "y")
				{
					childrow[m_Columns->m_col_useflags]	= true;
					childrow[m_Columns->m_col_mbd]		= reader.get_attribute("mbd");
					childrow[m_Columns->m_col_flags]	= reader.get_attribute("flags");
					childrow[m_Columns->m_col_cmp]		= reader.get_attribute("cmp");
					childrow[m_Columns->m_col_subcmp]	= reader.get_attribute("subcmp");
					childrow[m_Columns->m_col_g]		= reader.get_attribute("g");
				}
			}
		}
	}
	catch(const std::exception& ex)
	{
		if(verbose)
			std::cout << sep << "No user defined formats found: " << user_formats << std::endl;
	}

	return 0;

}
